@@ -419,6 +419,23 @@ class MultisetPartitionTraverser():
         self.k1 = 0
         self.k2 = 0
         self.p1 = 0
+        self.pstack = None
+        self.f = None
+        self.lpart = 0
+        self.discarded = 0
+        # dp_stack is list of lists of (part_key, start_count) pairs
+        self.dp_stack = []
+
+        # dp_map is map part_key-> count, where count represents the
+        # number of multiset which are descendants of a part with this
+        # key, **or any of its decrements**
+
+        # Thus, when we find a part in the map, we add its count
+        # value to the running total, cut off the enumeration, and
+        # backtrack
+
+        if not hasattr(self, 'dp_map'):
+            self.dp_map = {}
 
     def db_trace(self, msg):
         """Useful for understanding/debugging the algorithms.  Not
@@ -521,7 +538,7 @@ class MultisetPartitionTraverser():
         decision is made to fail, it must be accurate, otherwise the
         enumeration will miss some partitions.  But, it is OK not to
         capture all the possible failures -- if a part is passed that
-        shouldn't be, the resulting too-large partitions are filtered
+        should not be, the resulting too-large partitions are filtered
         by the enumeration one level up.  However, as is usual in
         constrained enumerations, failing early is advantageous.
 
@@ -607,7 +624,7 @@ class MultisetPartitionTraverser():
         """
 
         if amt == 1:
-            # In this case we always need to increment, *before*
+            # In this case we always need to decrement, *before*
             # enforcing the "sufficient unallocated multiplicity"
             # constraint.  Easiest for this is just to call the
             # regular decrement method.
@@ -651,7 +668,7 @@ class MultisetPartitionTraverser():
         Parameters
         ==========
 
-         part
+        part
             part to be decremented (topmost part on the stack)
 
         ub
@@ -757,7 +774,7 @@ class MultisetPartitionTraverser():
         See Also
         ========
 
-        multiset_partitions_taocp():
+        multiset_partitions_taocp:
             which provides the same result as this method, but is
             about twice as fast.  Hence, enum_all is primarily useful
             for testing.  Also see the function for a discussion of
@@ -825,18 +842,15 @@ class MultisetPartitionTraverser():
             return
         self._initialize_enumeration(multiplicities)
         while True:
-            good_partition = True
             while self.spread_part_multiplicity():
-                self.db_trace("spread 1")
+                self.db_trace('spread 1')
                 if self.lpart >= ub:
                     self.discarded += 1
-                    good_partition = False
-                    self.db_trace("  Discarding")
+                    self.db_trace('  Discarding')
                     self.lpart = ub - 2
                     break
-
-            # M4  Visit a partition
-            if good_partition:
+            else:
+                # M4  Visit a partition
                 state = [self.f, self.lpart, self.pstack]
                 yield state
 
@@ -1076,20 +1090,10 @@ class MultisetPartitionTraverser():
         """
         # number of partitions so far in the enumeration
         self.pcount = 0
+
         # dp_stack is list of lists of (part_key, start_count) pairs
         self.dp_stack = []
 
-        # dp_map is map part_key-> count, where count represents the
-        # number of multiset which are descendants of a part with this
-        # key, **or any of its decrements**
-
-        # Thus, when we find a part in the map, we add its count
-        # value to the running total, cut off the enumeration, and
-        # backtrack
-
-        if not hasattr(self, 'dp_map'):
-            self.dp_map = {}
-
         self._initialize_enumeration(multiplicities)
         pkey = part_key(self.top_part())
         self.dp_stack.append([(pkey, 0), ])
