@@ -1,6 +1,5 @@
 from __future__ import annotations
 from itertools import product
-from typing import Type
 
 from sympy.core.add import Add
 from sympy.core.assumptions import StdFactKB
@@ -28,11 +27,11 @@ class Vector(BasisDependent):
     is_Vector = True
     _op_priority = 12.0
 
-    _expr_type = None  # type: Type[Vector]
-    _mul_func = None  # type: Type[Vector]
-    _add_func = None  # type: Type[Vector]
-    _zero_func = None  # type: Type[Vector]
-    _base_func = None  # type: Type[Vector]
+    _expr_type: type[Vector]
+    _mul_func: type[Vector]
+    _add_func: type[Vector]
+    _zero_func: type[Vector]
+    _base_func: type[Vector]
     zero: VectorZero
 
     @property
